@@ -10,6 +10,7 @@ from .cedar import (
     Authorizer,
     Policy,
     PolicySet,
+    SchemaFragment,
     Schema,
     ValidationError,
     ValidationResult,
@@ -29,6 +30,7 @@ __all__ = [
     "Authorizer",
     "Policy",
     "PolicySet",
+    "SchemaFragment",
     "Schema",
     "ValidationError",
     "ValidationResult",
